# BuzzPerl.pm 0.3
# 2004 Nov 30; Kenward Bradley
# reads, writes, manipulates a Jeskola Buzz data file (BMX, BMW)
# The BuzzPerl.pm file must go in Perl's lib folder (usually "c:\perl\lib\")
# http://sourceforge.net/projects/buzzperl/

package 	BuzzPerl;
require		Exporter;

our @ISA		= qw(Exporter);
our @EXPORT		= qw(%buzz ReadBuzzFile WriteBuzzFile dumpbuzz Normalize $BuzzPerlVerbose);	# Symbols exported by default
our @EXPORT_OK	= qw(PrintSection PrintDword PrintRemainer);				# Symbols exported on request
our $VERSION	= 0.2;														# Version

use strict;
use Data::Dumper;			# to get detailed variable info

our $BuzzPerlVerbose;		# set this to 1 if we are debugging
our %buzz;					# this hash holds all the buzz file data

#print "BuzzPerl verbose level: $BuzzPerlVerbose\n";
return 1;

# high level subroutines
sub ReadBuzzFile ($);
sub WriteBuzzFile ($);

# low level subroutines
sub dumpbuzz($);
sub PrintSection;
sub PrintDword;
sub PrintRemainer;
sub PackWord;
sub PackDword;
sub PackByte;
sub PackAsciiz;
sub GetSectionRaw;
sub GetPara;
sub GetAsciiz;
sub GetByte;
sub GetX;
sub GetSection;
sub GetFloat;
sub GetDword;
sub GetWord;
sub GetHex;
sub Normalize;

# ===== High Level Subs ---------------------------------------------------------------------
sub ReadBuzzFile ($) {
my $buzzinfile = @_[0];

open (buzzfile, $buzzinfile) or die "Can't open \"$buzzinfile\"\n";

# ----- READ HEADER -----
if (GetSection ne "Buzz") {die "This is not a valid Buzz file.\n";}
$buzz{numsections} = GetDword;
for my $count (1..$buzz{numsections}) {
  #reads in the header
  $buzz{sections}[$count]= GetSection;
  $buzz{$buzz{sections}[$count]}{offset}=GetDword;
  $buzz{$buzz{sections}[$count]}{size}=GetDword;
	}

#  ----- READ SECTIONS -----

#  -- SEQU --
if (exists $buzz{SEQU}) {
  my $counts;
  my $counte;
  seek buzzfile,$buzz{SEQU}{offset},0;
  $buzz{SEQU}{songend}   = GetDword;
  $buzz{SEQU}{loopbegin} = GetDword;
  $buzz{SEQU}{loopend}   = GetDword;
  $buzz{SEQU}{numseq}    = GetWord;
  for $counts (1..$buzz{SEQU}{numseq}) {
    $buzz{SEQU}{seq}[$counts]{machindex}     = GetWord;
    $buzz{SEQU}{seq}[$counts]{numevents}     = GetDword;
    $buzz{SEQU}{seq}[$counts]{byteseventpos} = GetByte;
    $buzz{SEQU}{seq}[$counts]{bytesevent}    = GetByte;
    if ($buzz{SEQU}{seq}[$counts]{byteseventpos} gt 1) {die "this program can't handle more then 112 patterns yet. byteseventpos=$buzz{SEQU}{seq}[$counts]{byteseventpos}";}
    if ($buzz{SEQU}{seq}[$counts]{bytesevent}    gt 1) {die "this program can't handle more then 112 patterns yet. bytesevent=$buzz{SEQU}{seq}[$counts]{bytesevent}";}
    #assumes byteseventpos and bytesevent both = 1 (or 0)
    #this will be true only when number of patterns < 112
    # -- get events
    for $counte (1..$buzz{SEQU}{seq}[$counts]{numevents}) {
      $buzz{SEQU}{seq}[$counts]{events}[$counte]{p}=GetByte;
      $buzz{SEQU}{seq}[$counts]{events}[$counte]{e}=GetByte;
      }
    }
  }

#  -- PARA --
# should be before MACH
if (exists $buzz{PARA}) {
  my $count;
  my $countp;
  seek buzzfile,$buzz{PARA}{offset},0;
  $buzz{PARA}{nummachines}=GetDword;                   #number of machines
  for $count (1..$buzz{PARA}{nummachines}) {
    $buzz{PARA}{machine}[$count]{name}=GetAsciiz;      #machine name
    $buzz{PARA}{machine}[$count]{type}=GetAsciiz;      #machine type
    $buzz{PARA}{machine}[$count]{numglobalp}=GetDword; #number of global parameter
    $buzz{PARA}{machine}[$count]{numtrackp}=GetDword;  #number of track parameters
    if ($BuzzPerlVerbose>=2) {
      print "global para ".$buzz{PARA}{machine}[$count]{numglobalp} ."\n";
      print "track para  ".$buzz{PARA}{machine}[$count]{numtrackp}  ."\n";
      }
    #get global parameters
    for $countp (1..$buzz{PARA}{machine}[$count]{numglobalp}) {
      $buzz{PARA}{machine}[$count]{global}[$countp]{type}     = GetByte;
      $buzz{PARA}{machine}[$count]{global}[$countp]{name}     = GetAsciiz;
      $buzz{PARA}{machine}[$count]{global}[$countp]{minvalue} = GetDword;
      $buzz{PARA}{machine}[$count]{global}[$countp]{maxvalue} = GetDword;
      $buzz{PARA}{machine}[$count]{global}[$countp]{novalue}  = GetDword;
      $buzz{PARA}{machine}[$count]{global}[$countp]{flags}    = GetDword;
      $buzz{PARA}{machine}[$count]{global}[$countp]{defvalue} = GetDword;
      }
    #get track parameters
    for $countp (1..$buzz{PARA}{machine}[$count]{numtrackp}) {
      $buzz{PARA}{machine}[$count]{track}[$countp]{type}     = GetByte;
      $buzz{PARA}{machine}[$count]{track}[$countp]{name}     = GetAsciiz;
      $buzz{PARA}{machine}[$count]{track}[$countp]{minvalue} = GetDword;
      $buzz{PARA}{machine}[$count]{track}[$countp]{maxvalue} = GetDword;
      $buzz{PARA}{machine}[$count]{track}[$countp]{novalue}  = GetDword;
      $buzz{PARA}{machine}[$count]{track}[$countp]{flags}    = GetDword;
      $buzz{PARA}{machine}[$count]{track}[$countp]{defvalue} = GetDword;
      }
    }
  }

#  -- MACH --
# should be after PARA
if ($BuzzPerlVerbose>=2) {print "===MACH SECTION===";}
if (exists $buzz{MACH}) {
  my $count;
  my $countp;
  my $countt;
  my $paratype;
  seek buzzfile,$buzz{MACH}{offset},0;

  $buzz{MACH}{nummachines}= GetWord; #get number of machines
  if ($BuzzPerlVerbose>=2) {print "num machines $buzz{MACH}{nummachines}\n";}
  for $count (1..$buzz{MACH}{nummachines}) {
    $buzz{MACH}{machine}[$count]{name}=GetAsciiz; #get name of machine
    if ($BuzzPerlVerbose>=2) {print "===MACHINE $buzz{MACH}{machine}[$count]{name}\n";}
    $buzz{MACH}{machine}[$count]{type}=GetByte; #get type (0 = master, 1 = generator, 2 = effect)
    # if machine is not Master, get dll name
    if (($buzz{MACH}{machine}[$count]{type} eq 1)
        or ($buzz{MACH}{machine}[$count]{type} eq 2)) {
        $buzz{MACH}{machine}[$count]{dll}=GetAsciiz;
        }
    $buzz{MACH}{machine}[$count]{coordx}=GetFloat;     #get X coord of machine
    $buzz{MACH}{machine}[$count]{coordy}=GetFloat;     #get Y coord of machine
    $buzz{MACH}{machine}[$count]{data}=GetX(GetDword); #get machine specific data
    $buzz{MACH}{machine}[$count]{numattr}=GetWord;     #number of attributes
    # get machine attributes
    for my $countAttr (1..$buzz{MACH}{machine}[$count]{numattr}) {
      $buzz{MACH}{machine}[$count]{attr}[$countAttr]{key}=GetAsciiz; #get key
      $buzz{MACH}{machine}[$count]{attr}[$countAttr]{value}=GetDword;#get value
      }
      
    #get parameters
    #get global
    if ($BuzzPerlVerbose>=2) {print "number of global parameters $buzz{PARA}{machine}[$count]{numglobalp}\n";}
    #state of global parameters
    for $countp (1..$buzz{PARA}{machine}[$count]{numglobalp}) {
      $paratype = $buzz{PARA}{machine}[$count]{global}[$countp]{type};
      if ($paratype eq 0) {$buzz{MACH}{machine}[$count]{global}[$countp] = GetByte;}
      if ($paratype eq 1) {$buzz{MACH}{machine}[$count]{global}[$countp] = GetByte;}
      if ($paratype eq 2) {$buzz{MACH}{machine}[$count]{global}[$countp] = GetByte;}
      if ($paratype eq 3) {$buzz{MACH}{machine}[$count]{global}[$countp] = GetWord;}
      if ($BuzzPerlVerbose>=2) {print "global value $buzz{MACH}{machine}[$count]{global}[$countp]\n";}
      }
    #get number of tracks
    $buzz{MACH}{machine}[$count]{numtracks}=GetWord;
    if ($BuzzPerlVerbose>=2) {print "numtracks $buzz{MACH}{machine}[$count]{numtracks}\n";}
    #get state of track parameters for each track
    for $countt (1..$buzz{MACH}{machine}[$count]{numtracks}) {  #count tracks
      for $countp (1..$buzz{PARA}{machine}[$count]{numtrackp}) { #count parameters
        $paratype = $buzz{PARA}{machine}[$count]{track}[$countp]{type};
        if ($paratype eq 0) {$buzz{MACH}{machine}[$count]{track}[$countt][$countp] = GetByte;}
        if ($paratype eq 1) {$buzz{MACH}{machine}[$count]{track}[$countt][$countp] = GetByte;}
        if ($paratype eq 2) {$buzz{MACH}{machine}[$count]{track}[$countt][$countp] = GetByte;}
        if ($paratype eq 3) {$buzz{MACH}{machine}[$count]{track}[$countt][$countp] = GetWord;}
        if ($BuzzPerlVerbose>=2) {print "track value $buzz{MACH}{machine}[$count]{track}[$countt][$countp]\n";}
        }
      }

    }
  }

#  -- PATT --
#should be after MACH and PARA sections
if (exists $buzz{PATT}) {
  if ($BuzzPerlVerbose>=2) {print "===PATTERNS SECTION===\n";}
  my $cm; #machine counter
  my $cp; #patterns counter
  my $ck; #ticks counter
  my $cr; #parameter counter
  my $ct; #tracks counter
  seek buzzfile,$buzz{PATT}{offset},0;
  for $cm (1..$buzz{MACH}{nummachines}){
    $buzz{PATT}{machine}[$cm]{numpatterns}   = GetWord;           #get number of patterns
    $buzz{PATT}{machine}[$cm]{numtracks}     = GetWord;           #get number of tracks
    if ($BuzzPerlVerbose>=2) {
      print "num patterns = $buzz{PATT}{machine}[$cm]{numpatterns}\n";
      print "  num tracks = $buzz{PATT}{machine}[$cm]{numtracks}\n";
      }
    for $cp (1..$buzz{PATT}{machine}[$cm]{numpatterns}){
      $buzz{PATT}{machine}[$cm]{patt}[$cp]{name} = GetAsciiz;     #get name of pattern
      $buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength} = GetWord; #get pattern length
      if ($BuzzPerlVerbose>=2) {
        print "  name    $buzz{PATT}{machine}[$cm]{patt}[$cp]{name}\n";
        print "  pat len $buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength}\n";
        }
if ($BuzzPerlVerbose>=2) {print "==global parameters\n";}	
      #get global parameters
      for $ck (1..$buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength}) {   #ticks loop
        for $cr (1..$buzz{PARA}{machine}[$cm]{numglobalp}) {            #parameters loop
          my $paratype = $buzz{PARA}{machine}[$cm]{global}[$cr]{type};
          if ($paratype eq 0) {$buzz{PATT}{g}[$cm][$cp][$ck][$cr] = GetByte;}
          if ($paratype eq 1) {$buzz{PATT}{g}[$cm][$cp][$ck][$cr] = GetByte;}
          if ($paratype eq 2) {$buzz{PATT}{g}[$cm][$cp][$ck][$cr] = GetByte;}
          if ($paratype eq 3) {$buzz{PATT}{g}[$cm][$cp][$ck][$cr] = GetWord;}
	      if ($BuzzPerlVerbose>=2) {
              print "=$buzz{PATT}{g}[$cm][$cp][$ck][$cr]; \t\t";
              print " (mach=$cm patt=$cp tick=$ck track=$ct para=$cr)\n";
	      }

          }
        }
      if ($BuzzPerlVerbose>=2) {print "==track parameters\n";}	
      #get track parameters
      for $ct (1..$buzz{PATT}{machine}[$cm]{numtracks}) {               #tracks loop
        for $ck (1..$buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength}) { #ticks loop
          for $cr (1..$buzz{PARA}{machine}[$cm]{numtrackp}) {           #parameters loop
            my $paratype = $buzz{PARA}{machine}[$cm]{track}[$cr]{type};
            if ($paratype eq 0) {$buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr] = GetByte;}
            if ($paratype eq 1) {$buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr] = GetByte;}
            if ($paratype eq 2) {$buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr] = GetByte;}
            if ($paratype eq 3) {$buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr] = GetWord;}
            if ($BuzzPerlVerbose>=2) {
              print "=$buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr]; \t\t";
              print " (mach=$cm patt=$cp tick=$ck track=$ct para=$cr)\n";
              }
            }
          }
        }
      }
     }
  }


#  -- BLAH --
if (exists $buzz{BLAH}) {
  seek buzzfile,$buzz{BLAH}{offset},0;
  read buzzfile,$buzz{BLAH}{text},GetDword;
  }

#  -- BVER --
if (exists $buzz{BVER}) {
  seek buzzfile,$buzz{BVER}{offset},0;
  $buzz{BVER}{text}=GetAsciiz;
  if ($BuzzPerlVerbose>=1) {print "$buzz{BVER}{text}\n";}
  }

#  -- CONN --
if (exists $buzz{CONN}) {
  my $cc;                               #connection count
  seek buzzfile,$buzz{CONN}{offset},0;
  $buzz{CONN}{numconn}             = GetWord; #get number of connections
  
  for $cc (1..$buzz{CONN}{numconn}) {
    $buzz{CONN}{c}[$cc]{source} = GetWord; #get source machine index number
    $buzz{CONN}{c}[$cc]{dest}   = GetWord; #get dest machine index number
    $buzz{CONN}{c}[$cc]{amp}    = GetWord; #get amp value
    $buzz{CONN}{c}[$cc]{pan}    = GetWord; #get pan value
    }
  }

#read in all sections as raw data
for my $count (1..$buzz{numsections}) {
  $buzz{raw}{$buzz{sections}[$count]}{data}=GetSectionRaw(
     $buzz{$buzz{sections}[$count]}{offset},
     $buzz{$buzz{sections}[$count]}{size});
  $buzz{raw}{$buzz{sections}[$count]}{offset}=$buzz{$buzz{sections}[$count]}{offset};
  $buzz{raw}{$buzz{sections}[$count]}{size}  =$buzz{$buzz{sections}[$count]}{size};

  }
}



sub WriteBuzzFile ($) {
#print "BuzzPerl verbose level: $BuzzPerlVerbose\n";

my $buzzoutfile = @_[0];

#----START PACK PATTERNS----
{
 if ($BuzzPerlVerbose>=1) {print ">$buzz{PATT}{machine}[2]{patt}[1]{name}<\n<";}

  my $cm; 		# machine counter
  my $cp; 		# patterns counter
  my $ck; 		# ticks counter
  my $cr; 		# parameter counter
  my $ct; 		# tracks counter
  my $raw=""; 	# pattern data is packed and added sequentially to this var

  for $cm (1..$buzz{MACH}{nummachines}){

    $raw .= PackWord($buzz{PATT}{machine}[$cm]{numpatterns});   #number of patterns
    $raw .= PackWord($buzz{PATT}{machine}[$cm]{numtracks});     #number of tracks

    for $cp (1..$buzz{PATT}{machine}[$cm]{numpatterns}){
      $raw .= PackAsciiz($buzz{PATT}{machine}[$cm]{patt}[$cp]{name});     #name of pattern
      $raw .= PackWord($buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength}); #pattern length

      #global parameters
      for $ck (1..$buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength}) {   #ticks loop
        for $cr (1..$buzz{PARA}{machine}[$cm]{numglobalp}) {            #parameters loop
          my $paratype = $buzz{PARA}{machine}[$cm]{global}[$cr]{type};
          if ($paratype eq 0) {$raw .= PackByte($buzz{PATT}{g}[$cm][$cp][$ck][$cr]);}
          if ($paratype eq 1) {$raw .= PackByte($buzz{PATT}{g}[$cm][$cp][$ck][$cr]);}
          if ($paratype eq 2) {$raw .= PackByte($buzz{PATT}{g}[$cm][$cp][$ck][$cr]);}
          if ($paratype eq 3) {$raw .= PackWord($buzz{PATT}{g}[$cm][$cp][$ck][$cr]);}
           }
        }
      #track parameters
      for $ct (1..$buzz{PATT}{machine}[$cm]{numtracks}) {               #tracks loop
        for $ck (1..$buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength}) { #ticks loop
          for $cr (1..$buzz{PARA}{machine}[$cm]{numtrackp}) {           #parameters loop
            my $paratype = $buzz{PARA}{machine}[$cm]{track}[$cr]{type};
            if ($paratype eq 0) {$raw .= PackByte($buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr]);}
            if ($paratype eq 1) {$raw .= PackByte($buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr]);}
            if ($paratype eq 2) {$raw .= PackByte($buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr]);}
            if ($paratype eq 3) {$raw .= PackWord($buzz{PATT}{t}[$cm][$cp][$ct][$ck][$cr]);}
            }
          }
        }
      }
    }

#insert new raw pattern data
$buzz{raw}{PATT}{data}=$raw;
$buzz{raw}{PATT}{size}=length($raw);
$buzz{raw}{PATT}{offset}=-1; #we dont know what this is yet, we will calc it later

}
#----END PACK PATTERNS----
{# start calculate dir (header, offset, size, etc)

#calc offsets
for my $count (1..$buzz{numsections}) {
  #if this is the first section then the offset is 8 + (31 * 12)
  #else the offset is size+offset of previous section
  if ($count eq 1) {$buzz{raw}{$buzz{sections}[$count]}{offset} = 8 + (31 * 12);}
    else {
      $buzz{raw}{$buzz{sections}[$count]}{offset}=
                $buzz{raw}{$buzz{sections}[$count-1]}{offset} +
                $buzz{raw}{$buzz{sections}[$count-1]}{size};
      }
  if ($BuzzPerlVerbose>=1) {print "name $buzz{sections}[$count] ";
	print "offset $buzz{raw}{$buzz{sections}[$count]}{offset} ";
	print "\tsize $buzz{raw}{$buzz{sections}[$count]}{size}\n";
	}
  }

#list sections and offsets
#for my $count (1..$buzz{numsections}) {
#  $buzz{raw}{$buzz{sections}[$count]}{data}=GetSectionRaw(
#  print "name $buzz{sections}[$count] ";
#  print "offset $buzz{raw}{$buzz{sections}[$count]}{offset} ";
#  print "\tsize $buzz{raw}{$buzz{sections}[$count]}{size}\n";
#  }

if ($BuzzPerlVerbose>=1) {print "\nFile pos at ".(tell buzzfile)."\n";}

}# start calculate dir (header, offset, size, etc)
# --- Save BMX/BMW file -------------------------------------------------
open (outfile, '>', $buzzoutfile) or die "Can't open \"$buzzoutfile\"\n";
binmode(outfile);

#save header
print outfile "Buzz";
print outfile PackDword($buzz{numsections});

#save dir entries
for my $count (1..$buzz{numsections}) {
  print outfile $buzz{sections}[$count];
  print outfile PackDword($buzz{raw}{$buzz{sections}[$count]}{offset});
  print outfile PackDword($buzz{raw}{$buzz{sections}[$count]}{size});
  }
#save remaining empty dir entries as 3 null dwords each
for my $count (1..(31-$buzz{numsections})) {
  print outfile PackDword(0);
  print outfile PackDword(0);
  print outfile PackDword(0);
  }
#save sections
for my $count (1..$buzz{numsections}) {
  print outfile $buzz{raw}{$buzz{sections}[$count]}{data};
  }

close outfile;
}

# --- Low Level Subs ---------------------------------------------------------------------
sub GetSectionRaw {
 	my $getit;
  # @_[0] (1st Arg) = position
  # @_[1] (2nd Arg) = size
  seek buzzfile, @_[0], 0 or die "at seek in GetSectionRaw. Pos=@_[0], Off=@_[1]\n";
	read(buzzfile, $getit, @_[1]) or die "at read in GetSectionRaw. Pos=@_[0], Off=@_[1]\n";
	return $getit;
  }
sub dumpbuzz($) {
#  $Data::Dumper::Indent = 1;
  $Data::Dumper::Terse = 2;          # don't output names where feasible
  $Data::Dumper::Indent = 1;         # pretty print
  
  if (@_[0] eq '') {@_[0]="BuzzPerlDataDump.txt";}
  open(DEBUGOUT, ">@_[0]") or die "Cant open dump outfile @_[0]\n";
  print DEBUGOUT Dumper(\%buzz);
  close DEBUGOUT;
  if ($BuzzPerlVerbose>=1) {print "Dumped to @_[0]\n";}
  }
sub GetPara {
  my %para;
  #$para{type}     = GetByte;
  #$para{name}     = GetAsciiz;
  #$para{minvalue} = GetDword;
  #$para{maxvalue} = GetDword;
  #$para{novalue}  = GetDword;
  #$para{flags}    = GetDword;
  #$para{defvalue} = GetDword;
  return (GetByte,GetAsciiz,GetDword,GetDword,GetDword,GetDword,GetDword);
  }
  
sub GetAsciiz {
  # gets a string terminated by a null
  my $getit;
  my $olddelim;
  $olddelim = $/;
  $/ = "\x00";
  $getit = <buzzfile>;
  $getit =~ s/\x00$//;
  $/ = $olddelim;
  return $getit;
  }

sub GetX {
	my $getit;
	read(buzzfile, $getit, @_[0]);
	return $getit;
  }

sub GetHex {
	my $getit;
	read(buzzfile, $getit, @_[0]);
	if ($BuzzPerlVerbose>=1) {
		print "@_[0] bytes: " . unpack "H*", $getit;
		print "\n";
		}
	return unpack "H*", $getit;
  }

sub GetWord {
	my $getit;
	read(buzzfile, $getit, 2);
	return unpack "S", $getit;
  }
sub PackDword {
	# @_[0] 1st parameter is data to be packed
	return pack "L", @_[0];
  }

sub PackWord {
	# @_[0] 1st parameter is data to be packed
	return pack "S", @_[0];
  }

sub PackByte {
	# @_[0] 1st parameter is data to be packed
	return pack "C", @_[0];
  }

sub PackAsciiz {
	# @_[0] 1st parameter is data to be packed
	return @_[0] . "\x00";
  }

sub GetFloat {
	my $getit;
	read(buzzfile, $getit, 4);
	return unpack "f", $getit;
  }

sub GetByte { #returns unsigned byte
	return unpack "C", getc buzzfile
  }

sub GetDword {
	my $getit;
	read(buzzfile, $getit, 4);
	return unpack "L", $getit;
	}

sub PrintDword {
	print "<dword>" . GetDword . "</dword>\n";
	}

sub GetSection {
	my $getit;
	read(buzzfile, $getit, 4);
	return $getit;
	}

sub PrintSection {
	print "<section>" . GetSection . "</section>\n";
	}

sub PrintRemainer {
	my $readin;
	while (not eof(buzzfile)) {
		read(buzzfile, $readin, 4);
		print $readin;
		}
	}

sub Normalize { 
# params: initial number, low end of initial range, high end of initial range, low end of new range, high end of new range
#         @_[0]           @_[1]                     @_[2]                      @_[3]                 @_[4]
# 
# returned: integer normalized into new range
	my $norm;
if (@_[2] == @_[1]) # we don't like any of your fancy "divide by zero" around these parts!
	{$norm=@_[4]}
	else {$norm = (@_[0] * ((@_[4] - @_[3]) / (@_[2] - @_[1])));}
	return @_[0];
  }