# buzztest 0.3
# 2004 Nov 30; Kenward Bradley
# example code using BuzzPerl.pm
# http://sourceforge.net/projects/buzzperl/

# Machines in template.bmx
# 1 Master
# 2 Bass
# 3 Dist
# 4 Filter
# 5 ErsKick
# 6 ErsSnare
# 7 ErsHihat


use BuzzPerl;
use strict;
use Data::Dumper; #to get detailed variable info

sub PrepareMaster;
sub pattbass;
sub pattkick;
sub pattsnare;


ReadBuzzFile("template.bmx");
PrepareMaster;							# define 0 patterns, 0 tracks for Master
pattbass;								# make some randomish patterns for bass
pattkick;
pattsnare;
WriteBuzzFile("outfile.bmx");



# --- subroutines --------------------------------------

sub PrepareMaster {
	# 1st machine is always master
	$buzz{PATT}{machine}[1]{numpatterns}   = 0;           #get number of patterns
	$buzz{PATT}{machine}[1]{numtracks}     = 0;           #get number of tracks
	}

sub pattbass {
  my $cm; #machine counter
  my $cp; #patterns counter
  my $ck; #ticks counter
  my $cr; #parameter counter
  my $ct; #tracks counter

# 2nd machine in default template.bmx is Jeskola Bass
$buzz{PATT}{machine}[2]{numpatterns}   = 4;           #get number of patterns
$buzz{PATT}{machine}[2]{numtracks}     = 1;           #get number of tracks

# start pattern loop
# for each pattern for this machine; setup pattern names and pattern length (in ticks)
for $cp (1..$buzz{PATT}{machine}[2]{numpatterns}) {
	$buzz{PATT}{machine}[2]{patt}[$cp]{name} = "Auto$cp"; 
	$buzz{PATT}{machine}[2]{patt}[$cp]{pattlength} = 16;
	
# SET GLOBAL VALUES
#$buzz{PATT}{g}[$machine][$patterns][$ticks][$parameter]
for $ck (1..$buzz{PATT}{machine}[2]{patt}[1]{pattlength}) {
    $buzz{PATT}{g}[2][$cp][$ck][1]=0xFF;                 # Cutoff
    $buzz{PATT}{g}[2][$cp][$ck][2]=0xFF;                 # Resonance
    }
  
	# SET TRACK VALUES
	#$buzz{PATT}{t}[$machine][$patterns][$ticks][$parameter]
	for $ct (1..$buzz{PATT}{machine}[2]{numtracks}) {        					# tracks loop
		for $ck (1..$buzz{PATT}{machine}[2]{patt}[1]{pattlength}) {           # ticks loop
			$buzz{PATT}{t}[2][$cp][$ct][$ck][1] = 0;         					# Waveform

# ---- Start of note assignment section		
			$buzz{PATT}{t}[2][$cp][$ct][$ck][2] = 0;       				 	# Note -- default to none
			if ((int(($ck-1)/2)==(($ck-1)/2)) && (rand() <= 0.35)) { 		 ### for every 2nd tick + 35%
				$buzz{PATT}{t}[2][$cp][$ct][$ck][2] = 0x21 + int(rand(0xC));} # chance, assign a random  
																				 # note in 2nd octave
			if ((int(($ck-1)/4)==(($ck-1)/4)) && (rand() <= 0.70)) { 		 ### for every 4th tick and 70% 
				$buzz{PATT}{t}[2][$cp][$ct][$ck][2] = 0x31 + int(rand(0xC));} # chance, assign a random 
																				 #note in 3rd octave
			if (($ck==1) || (int(($ck-1)/16)==(($ck-1)/16))) { 				 ### for the 1st or 16th tick 
				$buzz{PATT}{t}[2][$cp][$ct][$ck][2] = 0x31 + int(rand(0xC));} # always, assign a random
																				 # note in 3rd octave
# ---- End of note assignment section		

			$buzz{PATT}{t}[2][$cp][$ct][$ck][3] = 255;       # Volume
			$buzz{PATT}{t}[2][$cp][$ct][$ck][4] = 0;         # Slide note
			$buzz{PATT}{t}[2][$cp][$ct][$ck][5] = 0;         # Slide lenght (sic)
		}
    }
	}
}


sub pattkick {
  my $cm=5; #machine number
  my $cp; #patterns counter
  my $ck; #ticks counter
  my $cr; #parameter counter
  my $ct; #tracks counter

# 2nd machine in default template.bmx is Jeskola Bass
$buzz{PATT}{machine}[$cm]{numpatterns}   = 2;           #set number of patterns
$buzz{PATT}{machine}[$cm]{numtracks}     = 1;           #set number of tracks

# start pattern loop
# for each pattern for this machine; setup pattern names and pattern length (in ticks)
for $cp (1..$buzz{PATT}{machine}[$cm]{numpatterns}) {
	$buzz{PATT}{machine}[$cm]{patt}[$cp]{name} = "Auto$cp"; 
	$buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength} = 16;
	
# SET GLOBAL VALUES
# no global parms used in ErsKick
#$buzz{PATT}{g}[$machine][$patterns][$ticks][$parameter]
#for $ck (1..$buzz{PATT}{machine}[2]{patt}[1]{pattlength}) {
#    $buzz{PATT}{g}[$cm][$cp][$ck][1]=0xFF;                 # 
#    $buzz{PATT}{g}[$cm][$cp][$ck][2]=0xFF;                 # 
#    }
  
	# SET TRACK VALUES
	#$buzz{PATT}{t}[$machine][$patterns][$ticks][$parameter]
	for $ct (1..$buzz{PATT}{machine}[$cm]{numtracks}) {        					# tracks loop
		for $ck (1..$buzz{PATT}{machine}[$cm]{patt}[1]{pattlength}) {             # ticks loop
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][1] = 0xFFFF;         			# StartFreq
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][2] = 0xFFFF;         			# EndFreq
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][3] = 0xFFFF;       			# Decay
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][4] = 0xFFFF;         			# FreqDecay
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][5] = 0xFFFF;         			# Thump
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][6] = 0xFF;         			# Gain

# ---- Start of Trig assignment section											Trig
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][7] = 0xFF;       				# Note -- default to none
			if ((int(($ck-1)/2)==(($ck-1)/2)) && (rand() <= 0.20)) { 		### for every 2nd tick + 20%
				$buzz{PATT}{t}[$cm][$cp][$ct][$ck][7] = 1;} 					# chance, Trig=1  
				
			if ((int(($ck-1)/4)==(($ck-1)/4)) && (rand() <= 0.50)) { 		### for every 4th tick and 50% 
				$buzz{PATT}{t}[$cm][$cp][$ct][$ck][7] = 1;} 					# chance, Trig=1 
				
			if (($ck==1) || (int(($ck-1)/16)==(($ck-1)/16))) { 				 ### for the 1st or 16th tick 
				$buzz{PATT}{t}[$cm][$cp][$ct][$ck][7] = 1;} 					# always, Trig=1

# ---- End of Trig assignment section		

		}
    }
	}
}

sub pattsnare {
  my $cm=6; #machine number
  my $cp; #patterns counter
  my $ck; #ticks counter
  my $cr; #parameter counter
  my $ct; #tracks counter
  my $k;  #ticks but starts at zero rather then $tk which starts at 1, nice for comparisions

# 2nd machine in default template.bmx is Jeskola Bass
$buzz{PATT}{machine}[$cm]{numpatterns}   = 2;           #set number of patterns
$buzz{PATT}{machine}[$cm]{numtracks}     = 1;           #set number of tracks

# start pattern loop
# for each pattern for this machine; setup pattern names and pattern length (in ticks)
for $cp (1..$buzz{PATT}{machine}[$cm]{numpatterns}) {
	$buzz{PATT}{machine}[$cm]{patt}[$cp]{name} = "Auto$cp"; 
	$buzz{PATT}{machine}[$cm]{patt}[$cp]{pattlength} = 16;
	
# SET GLOBAL VALUES
# no global parms used in this machine
#$buzz{PATT}{g}[$machine][$patterns][$ticks][$parameter]
#for $ck (1..$buzz{PATT}{machine}[2]{patt}[1]{pattlength}) {
#    $buzz{PATT}{g}[$cm][$cp][$ck][1]=0xFF;                 # 
#    $buzz{PATT}{g}[$cm][$cp][$ck][2]=0xFF;                 # 
#    }
  
	# SET TRACK VALUES
	#$buzz{PATT}{t}[$machine][$patterns][$ticks][$parameter]
	for $ct (1..$buzz{PATT}{machine}[$cm]{numtracks}) {        					# tracks loop
		for $ck (1..$buzz{PATT}{machine}[$cm]{patt}[1]{pattlength}) {             # ticks loop
			$k = $ck - 1;
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][1] = 0xFFFF;         			# ThumpFreq
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][2] = 0xFFFF;         			# ThumpDecay
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][3] = 0xFFFF;       			# NoiseDecay
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][4] = 0xFFFF;         			# Thump
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][5] = 0xFF;         			# Gain

# ---- Start of Trig assignment section											Trig
			$buzz{PATT}{t}[$cm][$cp][$ct][$ck][6] = 0xFF;       				# Trig -- default to none
			
			if ((($k==2) || ($k==4) || ($k==6) ) && (rand() <= 0.70)) { 		### tick 2, 4, 6 + 70%
				$buzz{PATT}{t}[$cm][$cp][$ct][$ck][6] = 1;} 					# chance, Trig=1  

			if (($k>=10) && ($k<=14) && (rand() <= 0.35)) { 					### tick >=10, <=14, 35%
				$buzz{PATT}{t}[$cm][$cp][$ct][$ck][6] = 1;} 					# chance, Trig=1  

			if (($ck-1==0) || (int(($ck-1)/8)==(($ck-1)/8))) { 				### for the 1st or 16th tick 
				$buzz{PATT}{t}[$cm][$cp][$ct][$ck][6] = 0xFF;} 				# always, Trig=none

# ---- End of Trig assignment section		

		}
    }
	}
}